﻿# ihm de configuration de la résolution de l'écran dans Winpe

Add-Type -AssemblyName "system.windows.forms"
Add-Type -AssemblyName "System.Drawing"
 

# la configuration sélectionnée
$global:x="none"

#region api
$code = @'
// but : get et set des paramètres d'affichage pour le seul monitor actif !
// limite : pour traiter plusieurs monitors, il faut inroduire la fonction EnumDisplayDevices

// pour compiler avec CSC.EXE si besoin
// c:\windows\Microsoft.NET\Framework\v4.0.30319\csc /target:exe /win32icon:D:\Utilisateurs\qjnf891\Desktop\icones\icone34.ico setvideo.txt

// surprise : avec Winpe, le bureau n'est pas adapté avec les nouvelles dimensions de l'écran
// pourtant MSDN précise :"When the display mode is changed dynamically, the WM_DISPLAYCHANGE message is sent to all running applications with the following message parameters."

// msdn et la fonction ChangeDisplaySettings
// https://msdn.microsoft.com/en-us/library/windows/desktop/dd183411(v=vs.85).aspx

// msdn et la fonction EnumDisplaySettings
// https://msdn.microsoft.com/en-us/library/windows/desktop/dd162611(v=vs.85).aspx

// un des nombreux sites contenant ce code
// http://poshcode.org/2001
// https://gist.github.com/mintsoft/22a5ae4cc68d3e51b2f2

// eventuellement à voir
// https://www.codeproject.com/articles/36664/changing-display-settings-programmatically#EnumDisplaySettings

// spécial : pour compiler du code cpp ( et non du c# )
// http://csharpening.net/?p=689

using System; 
using System.Runtime.InteropServices; 
 
namespace Resolution 
{ 
 
    [StructLayout(LayoutKind.Sequential)] 
    public struct DEVMODE1 
    { 
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)] 
        public string dmDeviceName; 
        public short dmSpecVersion; 
        public short dmDriverVersion; 
        public short dmSize; 
        public short dmDriverExtra; 
        public int dmFields; 
 
        public short dmOrientation; 
        public short dmPaperSize; 
        public short dmPaperLength; 
        public short dmPaperWidth; 
 
        public short dmScale; 
        public short dmCopies; 
        public short dmDefaultSource; 
        public short dmPrintQuality; 
        public short dmColor; 
        public short dmDuplex; 
        public short dmYResolution; 
        public short dmTTOption; 
        public short dmCollate; 
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)] 
        public string dmFormName; 
        public short dmLogPixels; 
        public short dmBitsPerPel; 
        public int dmPelsWidth; 
        public int dmPelsHeight; 
 
        public int dmDisplayFlags; 
        public int dmDisplayFrequency; 
 
        public int dmICMMethod; 
        public int dmICMIntent; 
        public int dmMediaType; 
        public int dmDitherType; 
        public int dmReserved1; 
        public int dmReserved2; 
 
        public int dmPanningWidth; 
        public int dmPanningHeight; 
    }; 
 
 
 
    class User_32 
    { 
        [DllImport("user32.dll")] 
        public static extern int EnumDisplaySettings(string deviceName, int modeNum, ref DEVMODE1 devMode); 
        [DllImport("user32.dll")] 
        public static extern int ChangeDisplaySettings(ref DEVMODE1 devMode, int flags); 
 
        public const int ENUM_CURRENT_SETTINGS = -1; 
        public const int CDS_UPDATEREGISTRY = 0x01; 
        public const int CDS_TEST = 0x02; 
        public const int DISP_CHANGE_SUCCESSFUL = 0; 
        public const int DISP_CHANGE_RESTART = 1; 
        public const int DISP_CHANGE_FAILED = -1; 
    } 
 
 
 
    public class PrmaryScreenResolution 
    { 
        static public string ChangeResolution(int width, int height) 
        { 
 
            DEVMODE1 dm = GetDevMode1(); 
 
            if (0 != User_32.EnumDisplaySettings(null, User_32.ENUM_CURRENT_SETTINGS, ref dm)) 
            { 
 
                dm.dmPelsWidth = width; 
                dm.dmPelsHeight = height; 
 
                int iRet = User_32.ChangeDisplaySettings(ref dm, User_32.CDS_TEST); 
 
                if (iRet == User_32.DISP_CHANGE_FAILED) 
                { 
                    return "Unable To Process Your Request. Sorry For This Inconvenience."; 
                } 
                else 
                { 
                    iRet = User_32.ChangeDisplaySettings(ref dm, User_32.CDS_UPDATEREGISTRY); 
                    switch (iRet) 
                    { 
                        case User_32.DISP_CHANGE_SUCCESSFUL: 
                            { 
                                return "Success"; 
                            } 
                        case User_32.DISP_CHANGE_RESTART: 
                            { 
                                return "You Need To Reboot For The Change To Happen.\n If You Feel Any Problem After Rebooting Your Machine\nThen Try To Change Resolution In Safe Mode."; 
                            } 
                        default: 
                            { 
                                return "Failed To Change The Resolution"; 
                            } 
                    } 
 
                } 
 
 
            } 
            else 
            { 
                return "Failed To Change The Resolution."; 
            } 
        } 
 
        private static DEVMODE1 GetDevMode1() 
        { 
            DEVMODE1 dm = new DEVMODE1(); 
            dm.dmDeviceName = new String(new char[32]); 
            dm.dmFormName = new String(new char[32]); 
            dm.dmSize = (short)Marshal.SizeOf(dm); 
            return dm; 
        } 
        static public string GetAllResolutions() 
        {
            String retour = new String(new char[150]); 
            retour = "";
            int i=0;
            DEVMODE1 dm = GetDevMode1(); 
            while( 0!= User_32.EnumDisplaySettings(null, i, ref dm)){
                retour = retour + "width="+(dm.dmPelsWidth).ToString() +"  height="+ (dm.dmPelsHeight).ToString();
                //retour = retour + "  BitsPerPel="+(dm.dmBitsPerPel).ToString() +"  DisplayFrequency="+ (dm.dmDisplayFrequency).ToString() + "\n\r";
                retour = retour + "  BitsPerPel="+(dm.dmBitsPerPel).ToString() +"  Frequency="+ (dm.dmDisplayFrequency).ToString() + "\n\r";
                dm = GetDevMode1(); 
                i++; 
            }
            return retour;
        }
        static public string GetCurrentResolution() 
        {
            String retour = new String(new char[132]); 
            DEVMODE1 dm = GetDevMode1();
            if (0 != User_32.EnumDisplaySettings(null, User_32.ENUM_CURRENT_SETTINGS, ref dm)) 
            { 
                retour = "width="+(dm.dmPelsWidth).ToString() +"  height="+ (dm.dmPelsHeight).ToString();
                retour = retour + "  BitsPerPel="+(dm.dmBitsPerPel).ToString() +"  DisplayFrequency="+ (dm.dmDisplayFrequency).ToString();
            }
            return retour;             
        }
        // pour compiler avec csc.exe : retirer les ___  et retirer le code PS
	    public static void ___Main( String[] args)
	    {
		    ChangeResolution(1024,768);
	    }
    } 
} 
'@

if (-not ("Resolution.PrmaryScreenResolution" -as [Type]))
{
    add-type -TypeDefinition $code
}
#endregion api
#region eventHandler
    # on ferme le script
    $button_Exit_OnClick=
    {
        $form1.Close()
    }
    $button_Apply_OnClick=
    {
        #$listBox.SelectedItem;
        $global:x=$listBox.SelectedItem
        #$label_CurrentConf.Text = $listBox.SelectedItem
        # pourquoi cette msgbox ne s'affiche pas ici ???
        "ChangeResolution"
        #width=1024  height=768  BitsPerPel=32  Frequency=60
        $t = ($listBox.SelectedItem -split "\s+") -split "="
        #width
        #800     $t[1]
        #height
        #600     $t[3]
        #BitsPerPel
        #32
        #[System.Windows.Forms.MessageBox]::Show("$($listBox.SelectedItem) $($t[1]) $($t[3])", "SelectedItem")
        if ([Resolution.PrmaryScreenResolution]::ChangeResolution($t[1],$t[3]))
        {
            $label_CurrentConf.Text  = [Resolution.PrmaryScreenResolution]::GetCurrentResolution() 

        }

        # Pour Winpe : Le system envoie un message WM_ChangeDisplay mais n'envoie pas le message WM_SettingChange
        # donc on force cet envoi
        $signature = @"
        [DllImport("user32.dll")]
        public static extern int SendMessage(int hWnd, int hMsg, int wParam, int lParam);
"@
        #Add the SendMessage function as a static method of a class
        $SendMessage = Add-Type -MemberDefinition $signature -Name "Win32SendMessage" -Namespace "Win32Functions" -PassThru
        $HWND_broadcast=0xffff
        $WM_SETTINGCHANGE = 0x1A

        "On envoie le message WM_SettingChange pour Winpe"
        "`$SendMessage::SendMessage($HWND_broadcast, $WM_SETTINGCHANGE,0,0)"
        $SendMessage::SendMessage($HWND_broadcast, $WM_SETTINGCHANGE,0,0)
    }

#endregion eventHandler
#region DeclarationDesVariables
    $form1 = New-Object System.Windows.Forms.Form
    $label_TexteCurrentConf = New-Object System.Windows.Forms.Label
    $label_CurrentConf = New-Object System.Windows.Forms.Label
    $label_TexteSelect = New-Object System.Windows.Forms.Label
    $button_Apply = New-Object System.Windows.Forms.Button
    $button_Exit = New-Object System.Windows.Forms.Button
    $listBox = New-Object System.Windows.Forms.ListBox
#endregion DeclarationDesVariables
#region SuspendLayout
    $listBox.SuspendLayout();
    $form1.SuspendLayout();
#endregion SuspendLayout
#region the controls
    #
    # label2
    # 
    $label_TexteCurrentConf.AutoSize = $true;
    $label_TexteCurrentConf.BorderStyle = [System.Windows.Forms.BorderStyle]::None
    $label_TexteCurrentConf.Location = New-Object System.Drawing.Point(35, 64);
    $label_TexteCurrentConf.Name = "label2";
    $label_TexteCurrentConf.Size = New-Object System.Drawing.Size(108, 15);
    $label_TexteCurrentConf.TabIndex = 1;
    $label_TexteCurrentConf.Text = "Current Configuration";
    # 
    # label3
    # 
    $label_CurrentConf.AutoSize = $true;
    $label_CurrentConf.BorderStyle = [System.Windows.Forms.BorderStyle]::Fixed3D;
    $label_CurrentConf.Location = New-Object System.Drawing.Point(195, 64);
    $label_CurrentConf.Name = "label3";
    $label_CurrentConf.Size = New-Object System.Drawing.Size(175, 15);
    $label_CurrentConf.TabIndex = 2;
    $label_CurrentConf.Text = "Width Height BitsPerPel Frequency";
    # 
    # label4
    # 
    $label_TexteSelect.AutoSize = $true;
    $label_TexteSelect.BorderStyle = [System.Windows.Forms.BorderStyle]::None;
    $label_TexteSelect.Location = New-Object System.Drawing.Point(35, 114);
    $label_TexteSelect.Name = "label4";
    $label_TexteSelect.Size = New-Object System.Drawing.Size(112, 15);
    $label_TexteSelect.TabIndex = 3;
    $label_TexteSelect.Text = "Select a configuration";
    #
    # listBox1
    # 
    $listBox.FormattingEnabled = $true;
    $listBox.Location = New-Object System.Drawing.Point(195, 114);
    $listBox.Name = "listBox1";
    $listBox.Size = New-Object System.Drawing.Size(355, 120);
    $listBox.TabIndex = 4;
    # 
    # button1
    # 
    $button_Apply.Location = New-Object System.Drawing.Point(61, 255);
    $button_Apply.Name = "button1";
    $button_Apply.Size = New-Object System.Drawing.Size(115, 23);
    $button_Apply.TabIndex = 5;
    $button_Apply.Text = "Apply";
    $button_Apply.UseVisualStyleBackColor = $true;
    $button_Apply.Add_Click($button_Apply_OnClick);
    $button_Apply.Anchor = "Bottom,Left";
    # 
    # button2
    # 
    $button_Exit.Location = New-Object System.Drawing.Point(236, 255);
    $button_Exit.Name = "button2";
    $button_Exit.Size = New-Object System.Drawing.Size(115, 23);
    $button_Exit.TabIndex = 6;
    $button_Exit.Text = "Exit";
    $button_Exit.UseVisualStyleBackColor = $true;
    $button_Exit.Add_Click($button_Exit_OnClick);
    $button_Exit.Anchor = "Bottom,Right";
    # 
    # Form1
    # 
    $form1.AutoScaleDimensions = New-Object System.Drawing.SizeF(0x6F, 0x13F);
    $form1.AutoScaleMode = [System.Windows.Forms.AutoScaleMode]::Font;
    $form1.ClientSize = New-Object System.Drawing.Size(580, 300);
    $form1.Controls.Add($button_Exit);
    $form1.Controls.Add($button_Apply);
    $form1.Controls.Add($listBox);
    $form1.Controls.Add($label_TexteSelect);
    $form1.Controls.Add($label_CurrentConf);
    $form1.Controls.Add($label_TexteCurrentConf);
    $form1.Name = "Form1";
    $form1.Text = "Configuration de la résolution de l'écran";
    $form1.StartPosition = [System.Windows.Forms.FormStartPosition]::CenterScreen;
    $form1.Font = New-Object System.Drawing.Font("Segoe UI", 9, [System.Drawing.FontStyle]::Regular, [System.Drawing.GraphicsUnit]::Point, 0);
    $form1.MinimumSize = $form1.Size
    $form1.MaximumSize = $form1.Size     # retrait sezz
    $form1.Topmost = $True 

    $form1.ResumeLayout($false);
    $form1.PerformLayout();

#endregion the controls

#region clavier
$form1.KeyPreview = $True
$form1.Add_KeyDown({if ($_.KeyCode -eq "Enter") 
    {$global:x=$listBox.SelectedItem;$form1.Close()}})
$form1.Add_KeyDown({if ($_.KeyCode -eq "Escape") 
    {$form1.Close()}})
#endregion clavier
#region get info de résolution
"GetCurrentResolution"
$label_CurrentConf.Text  = [Resolution.PrmaryScreenResolution]::GetCurrentResolution() 

"GetAllResolutions"
$AllResolutions = [Resolution.PrmaryScreenResolution]::GetAllResolutions()

foreach ($Resolution in ($AllResolutions -split "`n`r")){
    if ($Resolution -ne "" ) {
        [void] $listBox.Items.Add("$Resolution")
    }
}

#endregion get info de résolution
$form1.Add_Shown({$form1.Activate()})

[void] $form1.ShowDialog()

"$global:x"